#
# Copyright (C) 2013 - 2016 Satoru SATOH <ssato @ redhat.com>
# License: MIT
#
"""anyconfig globals.
"""
import anyconfig.init


PACKAGE = "anyconfig"
AUTHOR = "Satoru SATOH <ssat@redhat.com>"
VERSION = "0.7.0"

LOGGER = anyconfig.init.LOGGER

# vim:sw=4:ts=4:et:
